/******************************************/
/*                                        */
/* Code Copyright (C) Terry Richards 1992 */
/*                                        */
/* FREE for personal use only. Commercial */
/* use forbidden without my express       */
/* written permission.                    */
/*                                        */
/******************************************/

#include <dir.h>
#include <filedial.h>
#include <string.h>

#define BUFFSIZE	3000
#define MAXINT		32767

_CLASSDEF(TBFileWindow)
_CLASSDEF(TBFileBuffer)
_CLASSDEF(TBScroller)
_CLASSDEF(TBSearchDialog)
_CLASSDEF(TBSearchStruct)

struct TBSearchStruct {

	char 	SearchText[81];
  	BOOL 	CaseSensitive;

	//All fields before here are the transfer buffer
	//for the search dialog

	char 	ReplaceText[81];
  	BOOL 	ReplaceAll;
	BOOL 	PromptOnReplace;

	//All fields before here are the transfer buffer
	//for the search/replace dialog

	//The rest of the fields are for use in the actual searching

	long	StartPos; //Not preserved - reset each time
	BOOL	Found;
	long	FoundAt; //Only Valid if Found is TRUE

			TBSearchStruct(); //Constructor - sets everything to NULL or FALSE

};

class TBFileWindow : public TWindow{

protected:

	char            FileName[MAXPATH];
	BOOL            IsDirty,IsNewFile,IsKB,DoubleClick,DidSearch;
	PTBFileBuffer	Anchor;
	HDC             DragDC;
	BOOL            ButtonDown;
	BOOL			HasFocus;
	long            CaretPos;
	long            HiStart,HiEnd;
	int				CaretWidth;
	long            Size;
	TBSearchStruct 	SearchStruct;

	void            DeleteBuffers();
	virtual void    SetScroller();
	virtual void    SetCaret();
	virtual void    Initialize();
	virtual BOOL    EditCopy();

	virtual void 	GetWindowClass(WNDCLASS _FAR & AWndClass);


public:

					TBFileWindow(PTWindowsObject AParent,LPSTR ATitle);
					~TBFileWindow();

	virtual BOOL    CanClose();
	virtual void    Paint(HDC DC,PAINTSTRUCT& PS);

	virtual void    WMLButtonDown(RTMessage Msg)    = [WM_FIRST + WM_LBUTTONDOWN];
	virtual void    WMLButtonUp(RTMessage Msg)      = [WM_FIRST + WM_LBUTTONUP];
	virtual void    WMMouseMove(RTMessage Msg)      = [WM_FIRST + WM_MOUSEMOVE];
	virtual void    WMLButtonDblClk(RTMessage Msg)	= [WM_FIRST + WM_LBUTTONDBLCLK ];
	virtual void    WMKeyDown(RTMessage Msg)        = [WM_FIRST + WM_KEYDOWN];
	virtual void    WMChar(RTMessage Msg)        	= [WM_FIRST + WM_CHAR];
	virtual void    WMKillFocus(RTMessage Msg)      = [WM_FIRST + WM_KILLFOCUS];
	virtual void    WMSetFocus(RTMessage Msg)       = [WM_FIRST + WM_SETFOCUS];

	virtual void    CMFileNew(RTMessage Msg)        = [CM_FIRST + CM_FILENEW];
	virtual void    CMFileOpen(RTMessage Msg)       = [CM_FIRST + CM_FILEOPEN];
	virtual void    CMFileSave(RTMessage Msg)       = [CM_FIRST + CM_FILESAVE];
	virtual void    CMFileSaveAs(RTMessage Msg)     = [CM_FIRST + CM_FILESAVEAS];
	virtual void    CMEditCut(RTMessage Msg)        = [CM_FIRST + CM_EDITCUT];
	virtual void    CMEditCopy(RTMessage Msg)       = [CM_FIRST + CM_EDITCOPY];
	virtual void    CMEditPaste(RTMessage Msg)      = [CM_FIRST + CM_EDITPASTE];
	virtual void    CMEditDelete(RTMessage Msg)     = [CM_FIRST + CM_EDITDELETE];
	virtual void    CMEditClear(RTMessage Msg)      = [CM_FIRST + CM_EDITCLEAR];
	virtual void    CMEditUndo(RTMessage Msg)		= [CM_FIRST + CM_EDITUNDO];
	virtual void 	CMEditFind(RTMessage Msg) 		= [CM_FIRST + CM_EDITFIND];
	virtual void 	CMEditFindNext(RTMessage Msg) 	= [CM_FIRST + CM_EDITFINDNEXT];
	virtual void 	CMEditReplace(RTMessage Msg) 	= [CM_FIRST + CM_EDITREPLACE];

	virtual void    OpenFile();
	virtual void    SaveFile();
	virtual void    SaveFileAs();
//  static PTStreamable build();

/*protected:

	TBFileWindow(StreamableInit) : TWindow(streamableInit) {};
	virtual void write( Ropstream );
	virtual Pvoid read( Ripstream ); */

/*private:

	const Pchar streamableName() const { return "TBFileWindow"; }  */

};

/*inline Ripstream operator >> ( Ripstream is, RTBFileWindow cl )
  { return is >> (RTStreamable)cl; }

inline Ripstream operator >> ( Ripstream is, RPTBFileWindow cl )
  { return is >> (RPvoid)cl; }

inline Ropstream operator << ( Ropstream os, RTBFileWindow cl )
  { return os << (RTStreamable)cl; }

inline Ropstream operator << ( Ropstream os, PTBFileWindow cl )
  { return os << (PTStreamable)cl; } */

class TBFileBuffer{

protected:

	unsigned char	Buffer[BUFFSIZE];
	unsigned int    Size;           //The number of characters actually in Buffer
	int             LineHeight;     //In windows units
	int             CharAveWidth;   //In windows units
	int             TabSize;        //In windows units
	int             cCR;            //The number of carriage returns in this buffer
	long            LongestLine;    //The number of characters in the longest line
	PTWindowsObject Parent;         //Could be a window or a dialog or whatever

	virtual void    SetStats();
	virtual long    LastChars();    //Return number of chars that belong to the next line
	virtual void	Split();
		
public:

	PTBFileBuffer	Next;    //A standard bi-directional pointer chain
	PTBFileBuffer	Prev;
	COLORREF		TextColor;
    COLORREF		BGColor;

					TBFileBuffer(PTWindowsObject AParent,PTBFileBuffer APrev=NULL);

	virtual long    Load(ifstream& is);   //Read buffer from file
	virtual void    UnLoad(ofstream& os); //Write buffer to file
	virtual void    Paint(HDC DC,TScroller *Scroller,long HiStart,long HiEnd,long CharsSoFar = 0,long YPos=0,BOOL First=TRUE);
	virtual long    XRange(long XSoFar=0);
	virtual long    YRange(long YSoFar=0);
	virtual void    BufferToClient(long BufferPos,HDC hdc,long& X,long& Y,long YSoFar=0,long XOffset=0);
	virtual long    ClientToBuffer(long X,long Y,HDC DC,long CharsSoFar=0,long XOffSet=0);
	virtual int     Height(long BufferPos);
	virtual long    NextBreak(long BufferPos,long BufferSoFar=0);
	virtual long    PrevBreak(long BufferPos,long BufferSoFar=0);
	virtual long    NextCR(long BufferPos,long BufferSoFar=0);
	virtual long    PrevCR(long BufferPos,long BufferSoFar=0);
	virtual void	CopyChar(char FAR *Dest,long BufferPos,long CharsToCopy=1);
	virtual void	DeleteAt(long BufferPos,long CharsToDelete=1);
	virtual void	InsertAt(long BufferPos,unsigned char Char);
	virtual void	InsertAt(long BufferPos,char* String);
	virtual void	Search(TBSearchStruct& SearchStruct);

};

//Tscrollers erase the whole window when you scroll a whole page.
//This creates an annoying "flash" effect if the current window has
//highlighted text. (it changes black->white->black). TBEditWindow
//redraws everything anyway so there is no need to erase. So we OOP
//it and overload the one proc that does that.

class TBScroller : public TScroller{

public:

					TBScroller(PTWindow TheWindow,int TheXUnit,int TheYUnit,long TheXRange,long TheYRange):
					TScroller(TheWindow,TheXUnit,TheYUnit,TheXRange,TheYRange){};

	virtual void 	ScrollTo(long X, long Y);

};

class TBSearchDialog:public TDialog{

public:

	TBSearchDialog(PTWindowsObject AParent,int ResourceId,PTBSearchStruct SearchStruct,PTModule AModule = NULL);

};



